@@ -25,15 +25,28 @@ add_fullscreen_control <- function(map, position = "top-right") {
     )
 
     if (inherits(map, "mapboxgl_proxy") || inherits(map, "maplibre_proxy")) {
-        proxy_class <- if (inherits(map, "mapboxgl_proxy")) "mapboxgl-proxy" else "maplibre-proxy"
-
-        map$session$sendCustomMessage(proxy_class, list(
-            id = map$id,
-            message = list(
-                type = "add_fullscreen_control",
-                position = position
-            )
-        ))
+        if (inherits(map, "mapboxgl_compare_proxy") || inherits(map, "maplibre_compare_proxy")) {
+            # For compare proxies
+            proxy_class <- if (inherits(map, "mapboxgl_compare_proxy")) "mapboxgl-compare-proxy" else "maplibre-compare-proxy"
+            map$session$sendCustomMessage(proxy_class, list(
+                id = map$id,
+                message = list(
+                    type = "add_fullscreen_control",
+                    position = position,
+                    map = map$map_side
+                )
+            ))
+        } else {
+            # For regular proxies
+            proxy_class <- if (inherits(map, "mapboxgl_proxy")) "mapboxgl-proxy" else "maplibre-proxy"
+            map$session$sendCustomMessage(proxy_class, list(
+                id = map$id,
+                message = list(
+                    type = "add_fullscreen_control",
+                    position = position
+                )
+            ))
+        }
     }
 
     map
@@ -75,21 +88,37 @@ add_navigation_control <- function(map,
         inherits(map, "mapboxgl_proxy"),
         inherits(map, "maplibre_proxy")
     )) {
-        proxy_class <- if (inherits(map, "mapboxgl_proxy")) {
-            "mapboxgl-proxy"
+        if (inherits(map, "mapboxgl_compare_proxy") || inherits(map, "maplibre_compare_proxy")) {
+            # For compare proxies
+            proxy_class <- if (inherits(map, "mapboxgl_compare_proxy")) "mapboxgl-compare-proxy" else "maplibre-compare-proxy"
+            map$session$sendCustomMessage(proxy_class, list(
+                id = map$id,
+                message = list(
+                    type = "add_navigation_control",
+                    options = nav_control,
+                    position = position,
+                    orientation = orientation,
+                    map = map$map_side
+                )
+            ))
         } else {
-            "maplibre-proxy"
+            # For regular proxies
+            proxy_class <- if (inherits(map, "mapboxgl_proxy")) {
+                "mapboxgl-proxy"
+            } else {
+                "maplibre-proxy"
+            }
+            
+            map$session$sendCustomMessage(proxy_class, list(
+                id = map$id,
+                message = list(
+                    type = "add_navigation_control",
+                    options = nav_control,
+                    position = position,
+                    orientation = orientation
+                )
+            ))
         }
-
-        map$session$sendCustomMessage(proxy_class, list(
-            id = map$id,
-            message = list(
-                type = "add_navigation_control",
-                options = nav_control,
-                position = position,
-                orientation = orientation
-            )
-        ))
     } else {
         if (is.null(map$x$navigation_control)) {
             map$x$navigation_control <- list()
@@ -107,6 +136,13 @@ add_navigation_control <- function(map,
 #' @param position The position of the control on the map (one of "top-left", "top-right", "bottom-left", "bottom-right").
 #' @param layers A vector of layer IDs to be included in the control. If NULL, all layers will be included.
 #' @param collapsible Whether the control should be collapsible.
+#' @param use_icon Whether to use a stacked layers icon instead of the "Layers" text when collapsed. Only applies when collapsible = TRUE.
+#' @param background_color The background color for the layers control; this will be the
+#'        color used for inactive layer items.
+#' @param active_color The background color for active layer items.
+#' @param hover_color The background color for layer items when hovered.
+#' @param active_text_color The text color for active layer items.
+#' @param inactive_text_color The text color for inactive layer items.
 #'
 #' @return The modified map object with the layers control added.
 #' @export
@@ -130,25 +166,24 @@ add_navigation_control <- function(map,
 #'         source = rds,
 #'         line_color = "pink"
 #'     ) |>
-#'     add_layers_control(collapsible = TRUE)
+#'     add_layers_control(
+#'         position = "top-left",
+#'         background_color = "#ffffff",
+#'         active_color = "#4a90e2"
+#'     )
 #' }
 add_layers_control <- function(map,
                                position = "top-left",
                                layers = NULL,
-                               collapsible = FALSE) {
+                               collapsible = TRUE,
+                               use_icon = TRUE,
+                               background_color = NULL,
+                               active_color = NULL,
+                               hover_color = NULL,
+                               active_text_color = NULL,
+                               inactive_text_color = NULL) {
     control_id <- paste0("layers-control-", as.hexmode(sample(1:1000000, 1)))
 
-    # Create the control container
-    control_html <- paste0(
-        '<nav id="',
-        control_id,
-        '" class="layers-control',
-        ifelse(collapsible, " collapsible", ""),
-        '" style="',
-        position,
-        ': 10px;"></nav>'
-    )
-
     # If layers is NULL, get the layers added by the user
     if (is.null(layers)) {
         layers <- unlist(lapply(map$x$layers, function(y) {
@@ -156,32 +191,66 @@ add_layers_control <- function(map,
         }))
     }
 
+    # Create custom colors object if any color options were specified
+    custom_colors <- NULL
+    if (!is.null(background_color) || !is.null(active_color) ||
+        !is.null(hover_color) || !is.null(inactive_text_color) || !is.null(active_text_color)) {
+        custom_colors <- list()
+        if (!is.null(background_color)) custom_colors$background <- background_color
+        if (!is.null(active_color)) custom_colors$active <- active_color
+        if (!is.null(hover_color)) custom_colors$hover <- hover_color
+        if (!is.null(inactive_text_color)) custom_colors$text <- inactive_text_color
+        if (!is.null(active_text_color)) custom_colors$activeText <- active_text_color
+    }
+
     # Add control to map
     if (inherits(map, "mapboxgl_proxy") ||
         inherits(map, "maplibre_proxy")) {
-        proxy_class <- if (inherits(map, "mapboxgl_proxy")) {
-            "mapboxgl-proxy"
+        if (inherits(map, "mapboxgl_compare_proxy") || inherits(map, "maplibre_compare_proxy")) {
+            # For compare proxies
+            proxy_class <- if (inherits(map, "mapboxgl_compare_proxy")) "mapboxgl-compare-proxy" else "maplibre-compare-proxy"
+            map$session$sendCustomMessage(proxy_class, list(
+                id = map$id,
+                message = list(
+                    type = "add_layers_control",
+                    control_id = control_id,
+                    position = position,
+                    layers = layers,
+                    collapsible = collapsible,
+                    use_icon = use_icon,
+                    custom_colors = custom_colors,
+                    map = map$map_side
+                )
+            ))
         } else {
-            "maplibre-proxy"
+            # For regular proxies
+            proxy_class <- if (inherits(map, "mapboxgl_proxy")) {
+                "mapboxgl-proxy"
+            } else {
+                "maplibre-proxy"
+            }
+            map$session$sendCustomMessage(proxy_class, list(
+                id = map$id,
+                message = list(
+                    type = "add_layers_control",
+                    control_id = control_id,
+                    position = position,
+                    layers = layers,
+                    collapsible = collapsible,
+                    use_icon = use_icon,
+                    custom_colors = custom_colors
+                )
+            ))
         }
-        map$session$sendCustomMessage(proxy_class, list(
-            id = map$id,
-            message = list(
-                type = "add_layers_control",
-                control_id = control_id,
-                position = position,
-                layers = layers,
-                collapsible = collapsible
-            )
-        ))
     } else {
         map$x$layers_control <- list(
             control_id = control_id,
             position = position,
             layers = layers,
-            collapsible = collapsible
+            collapsible = collapsible,
+            use_icon = use_icon,
+            custom_colors = custom_colors
         )
-        map$x$control_html <- control_html
     }
 
     return(map)
@@ -196,15 +265,28 @@ add_layers_control <- function(map,
 clear_controls <- function(map) {
     if (inherits(map, "mapboxgl_proxy") ||
         inherits(map, "maplibre_proxy")) {
-        proxy_class <- if (inherits(map, "mapboxgl_proxy")) {
-            "mapboxgl-proxy"
+        if (inherits(map, "mapboxgl_compare_proxy") || inherits(map, "maplibre_compare_proxy")) {
+            # For compare proxies
+            proxy_class <- if (inherits(map, "mapboxgl_compare_proxy")) "mapboxgl-compare-proxy" else "maplibre-compare-proxy"
+            map$session$sendCustomMessage(proxy_class, list(
+                id = map$id,
+                message = list(
+                    type = "clear_controls",
+                    map = map$map_side
+                )
+            ))
         } else {
-            "maplibre-proxy"
+            # For regular proxies
+            proxy_class <- if (inherits(map, "mapboxgl_proxy")) {
+                "mapboxgl-proxy"
+            } else {
+                "maplibre-proxy"
+            }
+            map$session$sendCustomMessage(proxy_class, list(
+                id = map$id,
+                message = list(type = "clear_controls")
+            ))
         }
-        map$session$sendCustomMessage(proxy_class, list(
-            id = map$id,
-            message = list(type = "clear_controls")
-        ))
     }
     return(map)
 }
@@ -240,15 +322,29 @@ add_scale_control <- function(map,
 
     if (inherits(map, "mapboxgl_proxy") ||
         inherits(map, "maplibre_proxy")) {
-        proxy_class <- if (inherits(map, "mapboxgl_proxy")) {
-            "mapboxgl-proxy"
+        if (inherits(map, "mapboxgl_compare_proxy") || inherits(map, "maplibre_compare_proxy")) {
+            # For compare proxies
+            proxy_class <- if (inherits(map, "mapboxgl_compare_proxy")) "mapboxgl-compare-proxy" else "maplibre-compare-proxy"
+            map$session$sendCustomMessage(proxy_class, list(
+                id = map$id,
+                message = list(
+                    type = "add_scale_control", 
+                    options = scale_control,
+                    map = map$map_side
+                )
+            ))
         } else {
-            "maplibre-proxy"
+            # For regular proxies
+            proxy_class <- if (inherits(map, "mapboxgl_proxy")) {
+                "mapboxgl-proxy"
+            } else {
+                "maplibre-proxy"
+            }
+            map$session$sendCustomMessage(proxy_class, list(
+                id = map$id,
+                message = list(type = "add_scale_control", options = scale_control)
+            ))
         }
-        map$session$sendCustomMessage(proxy_class, list(
-            id = map$id,
-            message = list(type = "add_scale_control", options = scale_control)
-        ))
     } else {
         if (is.null(map$x$scale_control)) {
             map$x$scale_control <- list()
@@ -307,22 +403,40 @@ add_draw_control <- function(map,
 
     if (inherits(map, "mapboxgl_proxy") ||
         inherits(map, "maplibre_proxy")) {
-        proxy_class <- if (inherits(map, "mapboxgl_proxy")) {
-            "mapboxgl-proxy"
+        if (inherits(map, "mapboxgl_compare_proxy") || inherits(map, "maplibre_compare_proxy")) {
+            # For compare proxies
+            proxy_class <- if (inherits(map, "mapboxgl_compare_proxy")) "mapboxgl-compare-proxy" else "maplibre-compare-proxy"
+            map$session$sendCustomMessage(proxy_class, list(
+                id = map$id,
+                message = list(
+                    type = "add_draw_control",
+                    position = position,
+                    options = options,
+                    freehand = freehand,
+                    simplify_freehand = simplify_freehand,
+                    orientation = orientation,
+                    map = map$map_side
+                )
+            ))
         } else {
-            "maplibre-proxy"
+            # For regular proxies
+            proxy_class <- if (inherits(map, "mapboxgl_proxy")) {
+                "mapboxgl-proxy"
+            } else {
+                "maplibre-proxy"
+            }
+            map$session$sendCustomMessage(proxy_class, list(
+                id = map$id,
+                message = list(
+                    type = "add_draw_control",
+                    position = position,
+                    options = options,
+                    freehand = freehand,
+                    simplify_freehand = simplify_freehand,
+                    orientation = orientation
+                )
+            ))
         }
-        map$session$sendCustomMessage(proxy_class, list(
-            id = map$id,
-            message = list(
-                type = "add_draw_control",
-                position = position,
-                options = options,
-                freehand = freehand,
-                simplify_freehand = simplify_freehand,
-                orientation = orientation
-            )
-        ))
     }
 
     map
@@ -369,9 +483,9 @@ add_draw_control <- function(map,
 #' }
 get_drawn_features <- function(map) {
     if (!shiny::is.reactive(map) &&
-        !inherits(map, c("mapboxgl", "mapboxgl_proxy"))) {
+        !inherits(map, c("mapboxgl", "mapboxgl_proxy", "maplibregl", "maplibre_proxy"))) {
         stop(
-            "Invalid map object. Expected mapboxgl or mapboxgl_proxy object within a Shiny context."
+            "Invalid map object. Expected mapboxgl, mapboxgl_proxy, maplibre or maplibre_proxy object within a Shiny context."
         )
     }
 
@@ -393,10 +507,10 @@ get_drawn_features <- function(map) {
     # Get the session object
     session <- shiny::getDefaultReactiveDomain()
 
-    if (inherits(map, "mapboxgl")) {
+    if (inherits(map, "mapboxgl") || inherits(map, "maplibregl")) {
         # Initial map object in Shiny
         map_id <- map$elementId
-    } else if (inherits(map, "mapboxgl_proxy")) {
+    } else if (inherits(map, "mapboxgl_proxy") || inherits(map, "maplibre_proxy")) {
         # Proxy object
         map_id <- map$id
     } else {
@@ -404,10 +518,23 @@ get_drawn_features <- function(map) {
     }
 
     # Send message to get drawn features
-    session$sendCustomMessage("mapboxgl-proxy", list(
-        id = map_id,
-        message = list(type = "get_drawn_features")
-    ))
+    if (inherits(map, "mapboxgl_compare_proxy") || inherits(map, "maplibre_compare_proxy")) {
+        # For compare proxies
+        proxy_class <- if (inherits(map, "mapboxgl_compare_proxy")) "mapboxgl-compare-proxy" else "maplibre-compare-proxy"
+        session$sendCustomMessage(proxy_class, list(
+            id = map_id,
+            message = list(
+                type = "get_drawn_features",
+                map = map$map_side
+            )
+        ))
+    } else {
+        # For regular proxies
+        session$sendCustomMessage("mapboxgl-proxy", list(
+            id = map_id,
+            message = list(type = "get_drawn_features")
+        ))
+    }
 
     # Wait for response
     features_json <- NULL
@@ -468,15 +595,29 @@ add_geocoder_control <- function(map,
 
     if (inherits(map, "mapboxgl_proxy") ||
         inherits(map, "maplibre_proxy")) {
-        proxy_class <- if (inherits(map, "mapboxgl_proxy")) {
-            "mapboxgl-proxy"
+        if (inherits(map, "mapboxgl_compare_proxy") || inherits(map, "maplibre_compare_proxy")) {
+            # For compare proxies
+            proxy_class <- if (inherits(map, "mapboxgl_compare_proxy")) "mapboxgl-compare-proxy" else "maplibre-compare-proxy"
+            map$session$sendCustomMessage(proxy_class, list(
+                id = map$id,
+                message = list(
+                    type = "add_geocoder_control", 
+                    options = geocoder_options,
+                    map = map$map_side
+                )
+            ))
         } else {
-            "maplibre-proxy"
+            # For regular proxies
+            proxy_class <- if (inherits(map, "mapboxgl_proxy")) {
+                "mapboxgl-proxy"
+            } else {
+                "maplibre-proxy"
+            }
+            map$session$sendCustomMessage(proxy_class, list(
+                id = map$id,
+                message = list(type = "add_geocoder_control", options = geocoder_options)
+            ))
         }
-        map$session$sendCustomMessage(proxy_class, list(
-            id = map$id,
-            message = list(type = "add_geocoder_control", options = geocoder_options)
-        ))
     } else {
         if (is.null(map$x$geocoder_control)) {
             map$x$geocoder_control <- list()
@@ -522,15 +663,29 @@ add_reset_control <- function(map,
 
     if (inherits(map, "mapboxgl_proxy") ||
         inherits(map, "maplibre_proxy")) {
-        proxy_class <- if (inherits(map, "mapboxgl_proxy")) {
-            "mapboxgl-proxy"
+        if (inherits(map, "mapboxgl_compare_proxy") || inherits(map, "maplibre_compare_proxy")) {
+            # For compare proxies
+            proxy_class <- if (inherits(map, "mapboxgl_compare_proxy")) "mapboxgl-compare-proxy" else "maplibre-compare-proxy"
+            map$session$sendCustomMessage(proxy_class, list(
+                id = map$id,
+                message = list(
+                    type = "add_reset_control", 
+                    options = reset_control,
+                    map = map$map_side
+                )
+            ))
         } else {
-            "maplibre-proxy"
+            # For regular proxies
+            proxy_class <- if (inherits(map, "mapboxgl_proxy")) {
+                "mapboxgl-proxy"
+            } else {
+                "maplibre-proxy"
+            }
+            map$session$sendCustomMessage(proxy_class, list(
+                id = map$id,
+                message = list(type = "add_reset_control", options = reset_control)
+            ))
         }
-        map$session$sendCustomMessage(proxy_class, list(
-            id = map$id,
-            message = list(type = "add_reset_control", options = reset_control)
-        ))
     } else {
         map$x$reset_control <- reset_control
     }
@@ -595,15 +750,29 @@ add_geolocate_control <- function(map,
 
     if (inherits(map, "mapboxgl_proxy") ||
         inherits(map, "maplibre_proxy")) {
-        proxy_class <- if (inherits(map, "mapboxgl_proxy")) {
-            "mapboxgl-proxy"
+        if (inherits(map, "mapboxgl_compare_proxy") || inherits(map, "maplibre_compare_proxy")) {
+            # For compare proxies
+            proxy_class <- if (inherits(map, "mapboxgl_compare_proxy")) "mapboxgl-compare-proxy" else "maplibre-compare-proxy"
+            map$session$sendCustomMessage(proxy_class, list(
+                id = map$id,
+                message = list(
+                    type = "add_geolocate_control", 
+                    options = geolocate_control,
+                    map = map$map_side
+                )
+            ))
         } else {
-            "maplibre-proxy"
+            # For regular proxies
+            proxy_class <- if (inherits(map, "mapboxgl_proxy")) {
+                "mapboxgl-proxy"
+            } else {
+                "maplibre-proxy"
+            }
+            map$session$sendCustomMessage(proxy_class, list(
+                id = map$id,
+                message = list(type = "add_geolocate_control", options = geolocate_control)
+            ))
         }
-        map$session$sendCustomMessage(proxy_class, list(
-            id = map$id,
-            message = list(type = "add_geolocate_control", options = geolocate_control)
-        ))
     } else {
         if (is.null(map$x$geolocate_control)) {
             map$x$geolocate_control <- list()
