@@ -234,170 +234,132 @@ HTMLWidgets.widget({
                         });
                     }
 
-                    // Add layers if provided
-                    if (x.layers) {
-                        x.layers.forEach(function (layer) {
-                            try {
-                                const layerConfig = {
-                                    id: layer.id,
-                                    type: layer.type,
-                                    source: layer.source,
-                                    layout: layer.layout || {},
-                                    paint: layer.paint || {},
+                    function add_my_layers(layer) {
+                        try {
+                            const layerConfig = {
+                                id: layer.id,
+                                type: layer.type,
+                                source: layer.source,
+                                layout: layer.layout || {},
+                                paint: layer.paint || {},
+                            };
+
+                            // Check if source is an object and set generateId if source type is 'geojson'
+                            if (
+                                typeof layer.source === "object" &&
+                                layer.source.type === "geojson"
+                            ) {
+                                layerConfig.source.generateId = true;
+                            } else if (typeof layer.source === "string") {
+                                // Handle string source if needed
+                                layerConfig.source = layer.source;
+                            }
+
+                            if (layer.source_layer) {
+                                layerConfig["source-layer"] =
+                                    layer.source_layer;
+                            }
+
+                            if (layer.slot) {
+                                layerConfig["slot"] = layer.slot;
+                            }
+
+                            if (layer.minzoom) {
+                                layerConfig["minzoom"] = layer.minzoom;
+                            }
+
+                            if (layer.maxzoom) {
+                                layerConfig["maxzoom"] = layer.maxzoom;
+                            }
+
+                            if (layer.filter) {
+                                layerConfig["filter"] = layer.filter;
+                            }
+
+                            if (layer.before_id) {
+                                map.addLayer(layerConfig, layer.before_id);
+                            } else {
+                                map.addLayer(layerConfig);
+                            }
+
+                            // Add popups or tooltips if provided
+                            if (layer.popup) {
+                                map.on("click", layer.id, function (e) {
+                                    const description =
+                                        e.features[0].properties[layer.popup];
+
+                                    new maplibregl.Popup()
+                                        .setLngLat(e.lngLat)
+                                        .setHTML(description)
+                                        .addTo(map);
+                                });
+
+                                // Change cursor to pointer when hovering over the layer
+                                map.on("mouseenter", layer.id, function () {
+                                    map.getCanvas().style.cursor = "pointer";
+                                });
+
+                                // Change cursor back to default when leaving the layer
+                                map.on("mouseleave", layer.id, function () {
+                                    map.getCanvas().style.cursor = "";
+                                });
+                            }
+
+                            if (layer.tooltip) {
+                                const tooltip = new maplibregl.Popup({
+                                    closeButton: false,
+                                    closeOnClick: false,
+                                });
+
+                                // Create a reference to the mousemove handler function.
+                                // We need to pass 'e', 'map', 'tooltip', and 'layer.tooltip' to onMouseMoveTooltip.
+                                const mouseMoveHandler = function (e) {
+                                    onMouseMoveTooltip(
+                                        e,
+                                        map,
+                                        tooltip,
+                                        layer.tooltip,
+                                    );
                                 };
 
-                                // Check if source is an object and set generateId if source type is 'geojson'
-                                if (
-                                    typeof layer.source === "object" &&
-                                    layer.source.type === "geojson"
-                                ) {
-                                    layerConfig.source.generateId = true;
-                                } else if (typeof layer.source === "string") {
-                                    // Handle string source if needed
-                                    layerConfig.source = layer.source;
+                                // Create a reference to the mouseleave handler function.
+                                // We need to pass 'map' and 'tooltip' to onMouseLeaveTooltip.
+                                const mouseLeaveHandler = function () {
+                                    onMouseLeaveTooltip(map, tooltip);
+                                };
+
+                                // Attach the named handler references, not anonymous functions.
+                                map.on("mousemove", layer.id, mouseMoveHandler);
+                                map.on(
+                                    "mouseleave",
+                                    layer.id,
+                                    mouseLeaveHandler,
+                                );
+
+                                // Store these handler references so you can remove them later if needed
+                                if (!window._mapboxHandlers) {
+                                    window._mapboxHandlers = {};
+                                }
+                                window._mapboxHandlers[layer.id] = {
+                                    mousemove: mouseMoveHandler,
+                                    mouseleave: mouseLeaveHandler,
+                                };
+                            }
+
+                            // Add hover effect if provided
+                            if (layer.hover_options) {
+                                const jsHoverOptions = {};
+                                for (const [key, value] of Object.entries(
+                                    layer.hover_options,
+                                )) {
+                                    const jsKey = key.replace(/_/g, "-");
+                                    jsHoverOptions[jsKey] = value;
                                 }
 
-                                if (layer.source_layer) {
-                                    layerConfig["source-layer"] =
-                                        layer.source_layer;
-                                }
+                                let hoveredFeatureId = null;
 
-                                if (layer.slot) {
-                                    layerConfig["slot"] = layer.slot;
-                                }
-
-                                if (layer.minzoom) {
-                                    layerConfig["minzoom"] = layer.minzoom;
-                                }
-
-                                if (layer.maxzoom) {
-                                    layerConfig["maxzoom"] = layer.maxzoom;
-                                }
-
-                                if (layer.filter) {
-                                    layerConfig["filter"] = layer.filter;
-                                }
-
-                                if (layer.before_id) {
-                                    map.addLayer(layerConfig, layer.before_id);
-                                } else {
-                                    map.addLayer(layerConfig);
-                                }
-
-                                // Add popups or tooltips if provided
-                                if (layer.popup) {
-                                    map.on("click", layer.id, function (e) {
-                                        const description =
-                                            e.features[0].properties[
-                                                layer.popup
-                                            ];
-
-                                        new maplibregl.Popup()
-                                            .setLngLat(e.lngLat)
-                                            .setHTML(description)
-                                            .addTo(map);
-                                    });
-
-                                    // Change cursor to pointer when hovering over the layer
-                                    map.on("mouseenter", layer.id, function () {
-                                        map.getCanvas().style.cursor =
-                                            "pointer";
-                                    });
-
-                                    // Change cursor back to default when leaving the layer
-                                    map.on("mouseleave", layer.id, function () {
-                                        map.getCanvas().style.cursor = "";
-                                    });
-                                }
-
-                                if (layer.tooltip) {
-                                    const tooltip = new maplibregl.Popup({
-                                        closeButton: false,
-                                        closeOnClick: false,
-                                    });
-
-                                    // Create a reference to the mousemove handler function.
-                                    // We need to pass 'e', 'map', 'tooltip', and 'layer.tooltip' to onMouseMoveTooltip.
-                                    const mouseMoveHandler = function (e) {
-                                        onMouseMoveTooltip(
-                                            e,
-                                            map,
-                                            tooltip,
-                                            layer.tooltip,
-                                        );
-                                    };
-
-                                    // Create a reference to the mouseleave handler function.
-                                    // We need to pass 'map' and 'tooltip' to onMouseLeaveTooltip.
-                                    const mouseLeaveHandler = function () {
-                                        onMouseLeaveTooltip(map, tooltip);
-                                    };
-
-                                    // Attach the named handler references, not anonymous functions.
-                                    map.on(
-                                        "mousemove",
-                                        layer.id,
-                                        mouseMoveHandler,
-                                    );
-                                    map.on(
-                                        "mouseleave",
-                                        layer.id,
-                                        mouseLeaveHandler,
-                                    );
-
-                                    // Store these handler references so you can remove them later if needed
-                                    if (!window._mapboxHandlers) {
-                                        window._mapboxHandlers = {};
-                                    }
-                                    window._mapboxHandlers[layer.id] = {
-                                        mousemove: mouseMoveHandler,
-                                        mouseleave: mouseLeaveHandler,
-                                    };
-                                }
-
-                                // Add hover effect if provided
-                                if (layer.hover_options) {
-                                    const jsHoverOptions = {};
-                                    for (const [key, value] of Object.entries(
-                                        layer.hover_options,
-                                    )) {
-                                        const jsKey = key.replace(/_/g, "-");
-                                        jsHoverOptions[jsKey] = value;
-                                    }
-
-                                    let hoveredFeatureId = null;
-
-                                    map.on("mousemove", layer.id, function (e) {
-                                        if (e.features.length > 0) {
-                                            if (hoveredFeatureId !== null) {
-                                                map.setFeatureState(
-                                                    {
-                                                        source:
-                                                            typeof layer.source ===
-                                                            "string"
-                                                                ? layer.source
-                                                                : layer.id,
-                                                        id: hoveredFeatureId,
-                                                    },
-                                                    { hover: false },
-                                                );
-                                            }
-                                            hoveredFeatureId = e.features[0].id;
-                                            map.setFeatureState(
-                                                {
-                                                    source:
-                                                        typeof layer.source ===
-                                                        "string"
-                                                            ? layer.source
-                                                            : layer.id,
-                                                    id: hoveredFeatureId,
-                                                },
-                                                { hover: true },
-                                            );
-                                        }
-                                    });
-
-                                    map.on("mouseleave", layer.id, function () {
+                                map.on("mousemove", layer.id, function (e) {
+                                    if (e.features.length > 0) {
                                         if (hoveredFeatureId !== null) {
                                             map.setFeatureState(
                                                 {
@@ -411,46 +373,82 @@ HTMLWidgets.widget({
                                                 { hover: false },
                                             );
                                         }
-                                        hoveredFeatureId = null;
-                                    });
+                                        hoveredFeatureId = e.features[0].id;
+                                        map.setFeatureState(
+                                            {
+                                                source:
+                                                    typeof layer.source ===
+                                                    "string"
+                                                        ? layer.source
+                                                        : layer.id,
+                                                id: hoveredFeatureId,
+                                            },
+                                            { hover: true },
+                                        );
+                                    }
+                                });
 
-                                    Object.keys(jsHoverOptions).forEach(
-                                        function (key) {
-                                            const originalPaint =
-                                                map.getPaintProperty(
-                                                    layer.id,
-                                                    key,
-                                                ) || layer.paint[key];
-                                            map.setPaintProperty(
+                                map.on("mouseleave", layer.id, function () {
+                                    if (hoveredFeatureId !== null) {
+                                        map.setFeatureState(
+                                            {
+                                                source:
+                                                    typeof layer.source ===
+                                                    "string"
+                                                        ? layer.source
+                                                        : layer.id,
+                                                id: hoveredFeatureId,
+                                            },
+                                            { hover: false },
+                                        );
+                                    }
+                                    hoveredFeatureId = null;
+                                });
+
+                                Object.keys(jsHoverOptions).forEach(
+                                    function (key) {
+                                        const originalPaint =
+                                            map.getPaintProperty(
                                                 layer.id,
                                                 key,
-                                                [
-                                                    "case",
-                                                    [
-                                                        "boolean",
-                                                        [
-                                                            "feature-state",
-                                                            "hover",
-                                                        ],
-                                                        false,
-                                                    ],
-                                                    jsHoverOptions[key],
-                                                    originalPaint,
-                                                ],
-                                            );
-                                        },
-                                    );
-                                }
-                            } catch (e) {
-                                console.error(
-                                    "Failed to add layer: ",
-                                    layer,
-                                    e,
+                                            ) || layer.paint[key];
+                                        map.setPaintProperty(layer.id, key, [
+                                            "case",
+                                            [
+                                                "boolean",
+                                                ["feature-state", "hover"],
+                                                false,
+                                            ],
+                                            jsHoverOptions[key],
+                                            originalPaint,
+                                        ]);
+                                    },
+                                );
+                            }
+                        } catch (e) {
+                            console.error("Failed to add layer: ", layer, e);
+                        }
+                    }
+                    if (x.h3j_sources) {
+                        x.h3j_sources.forEach(async function (source) {
+                            await map.addH3JSource(source.id, {
+                                data: source.url,
+                            });
+
+                            // A bit hacky?
+                            if (x.layers) {
+                                x.layers.forEach((layer) =>
+                                    add_my_layers(layer),
                                 );
                             }
                         });
                     }
 
+                    // Add layers if provided
+                    if (x.layers) {
+                        x.layers.forEach((layer) => add_my_layers(layer));
+                    }
+
                     // Apply setFilter if provided
                     if (x.setFilter) {
                         x.setFilter.forEach(function (filter) {
@@ -900,9 +898,59 @@ HTMLWidgets.widget({
                             ? "layers-control collapsible"
                             : "layers-control";
                         layersControl.style.position = "absolute";
-                        layersControl.style[
-                            x.layers_control.position || "top-right"
-                        ] = "10px";
+
+                        // Set the position correctly - fix position bug by using correct CSS positioning
+                        const position =
+                            x.layers_control.position || "top-left";
+                        if (position === "top-left") {
+                            layersControl.style.top = "10px";
+                            layersControl.style.left = "10px";
+                        } else if (position === "top-right") {
+                            layersControl.style.top = "10px";
+                            layersControl.style.right = "10px";
+                        } else if (position === "bottom-left") {
+                            layersControl.style.bottom = "10px";
+                            layersControl.style.left = "10px";
+                        } else if (position === "bottom-right") {
+                            layersControl.style.bottom = "40px";
+                            layersControl.style.right = "10px";
+                        }
+
+                        // Apply custom colors if provided
+                        if (x.layers_control.custom_colors) {
+                            const colors = x.layers_control.custom_colors;
+
+                            // Create a style element for custom colors
+                            const styleEl = document.createElement("style");
+                            let css = "";
+
+                            if (colors.background) {
+                                css += `#${x.layers_control.control_id} { background-color: ${colors.background}; }\n`;
+                            }
+
+                            if (colors.text) {
+                                css += `#${x.layers_control.control_id} a { color: ${colors.text}; }\n`;
+                            }
+
+                            if (colors.active) {
+                                css += `#${x.layers_control.control_id} a.active { background-color: ${colors.active}; }\n`;
+                                css += `#${x.layers_control.control_id} .toggle-button { background-color: ${colors.active}; }\n`;
+                            }
+
+                            if (colors.activeText) {
+                                css += `#${x.layers_control.control_id} a.active { color: ${colors.activeText}; }\n`;
+                                css += `#${x.layers_control.control_id} .toggle-button { color: ${colors.activeText}; }\n`;
+                            }
+
+                            if (colors.hover) {
+                                css += `#${x.layers_control.control_id} a:hover { background-color: ${colors.hover}; }\n`;
+                                css += `#${x.layers_control.control_id} .toggle-button:hover { background-color: ${colors.hover}; }\n`;
+                            }
+
+                            styleEl.textContent = css;
+                            document.head.appendChild(styleEl);
+                        }
+
                         el.appendChild(layersControl);
 
                         const layersList = document.createElement("div");
@@ -924,7 +972,25 @@ HTMLWidgets.widget({
                             link.id = layerId;
                             link.href = "#";
                             link.textContent = layerId;
-                            link.className = "active";
+
+                            // Check if the layer visibility is set to "none" initially
+                            const initialVisibility = map.getLayoutProperty(
+                                layerId,
+                                "visibility",
+                            );
+                            link.className =
+                                initialVisibility === "none" ? "" : "active";
+
+                            // Also hide any associated legends if the layer is initially hidden
+                            if (initialVisibility === "none") {
+                                const associatedLegends =
+                                    document.querySelectorAll(
+                                        `.mapboxgl-legend[data-layer-id="${layerId}"]`,
+                                    );
+                                associatedLegends.forEach((legend) => {
+                                    legend.style.display = "none";
+                                });
+                            }
 
                             // Show or hide layer when the toggle is clicked
                             link.onclick = function (e) {
@@ -945,6 +1011,15 @@ HTMLWidgets.widget({
                                         "none",
                                     );
                                     this.className = "";
+
+                                    // Hide associated legends
+                                    const associatedLegends =
+                                        document.querySelectorAll(
+                                            `.mapboxgl-legend[data-layer-id="${clickedLayer}"]`,
+                                        );
+                                    associatedLegends.forEach((legend) => {
+                                        legend.style.display = "none";
+                                    });
                                 } else {
                                     this.className = "active";
                                     map.setLayoutProperty(
@@ -952,6 +1027,15 @@ HTMLWidgets.widget({
                                         "visibility",
                                         "visible",
                                     );
+
+                                    // Show associated legends
+                                    const associatedLegends =
+                                        document.querySelectorAll(
+                                            `.mapboxgl-legend[data-layer-id="${clickedLayer}"]`,
+                                        );
+                                    associatedLegends.forEach((legend) => {
+                                        legend.style.display = "";
+                                    });
                                 }
                             };
 
@@ -962,7 +1046,25 @@ HTMLWidgets.widget({
                         if (x.layers_control.collapsible) {
                             const toggleButton = document.createElement("div");
                             toggleButton.className = "toggle-button";
-                            toggleButton.textContent = "Layers";
+
+                            // Use stacked layers icon instead of text if requested
+                            if (x.layers_control.use_icon) {
+                                // Add icon-only class to the control for compact styling
+                                layersControl.classList.add("icon-only");
+
+                                // More GIS-like layers stack icon
+                                toggleButton.innerHTML = `<svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round">
+                                    <polygon points="12 2 2 7 12 12 22 7 12 2"></polygon>
+                                    <polyline points="2 17 12 22 22 17"></polyline>
+                                    <polyline points="2 12 12 17 22 12"></polyline>
+                                </svg>`;
+                                toggleButton.style.display = "flex";
+                                toggleButton.style.alignItems = "center";
+                                toggleButton.style.justifyContent = "center";
+                            } else {
+                                toggleButton.textContent = "Layers";
+                            }
+
                             toggleButton.onclick = function () {
                                 layersControl.classList.toggle("open");
                             };
@@ -1675,7 +1777,57 @@ if (HTMLWidgets.shinyMode) {
                     ? "layers-control collapsible"
                     : "layers-control";
                 layersControl.style.position = "absolute";
-                layersControl.style[message.position || "top-right"] = "10px";
+
+                // Set the position correctly
+                const position = message.position || "top-left";
+                if (position === "top-left") {
+                    layersControl.style.top = "10px";
+                    layersControl.style.left = "10px";
+                } else if (position === "top-right") {
+                    layersControl.style.top = "10px";
+                    layersControl.style.right = "10px";
+                } else if (position === "bottom-left") {
+                    layersControl.style.bottom = "10px";
+                    layersControl.style.left = "10px";
+                } else if (position === "bottom-right") {
+                    layersControl.style.bottom = "40px";
+                    layersControl.style.right = "10px";
+                }
+
+                // Apply custom colors if provided
+                if (message.custom_colors) {
+                    const colors = message.custom_colors;
+
+                    // Create a style element for custom colors
+                    const styleEl = document.createElement("style");
+                    let css = "";
+
+                    if (colors.background) {
+                        css += `#${message.control_id} { background-color: ${colors.background}; }\n`;
+                    }
+
+                    if (colors.text) {
+                        css += `#${message.control_id} a { color: ${colors.text}; }\n`;
+                    }
+
+                    if (colors.active) {
+                        css += `#${message.control_id} a.active { background-color: ${colors.active}; }\n`;
+                        css += `#${message.control_id} .toggle-button { background-color: ${colors.active}; }\n`;
+                    }
+
+                    if (colors.activeText) {
+                        css += `#${message.control_id} a.active { color: ${colors.activeText}; }\n`;
+                        css += `#${message.control_id} .toggle-button { color: ${colors.activeText}; }\n`;
+                    }
+
+                    if (colors.hover) {
+                        css += `#${message.control_id} a:hover { background-color: ${colors.hover}; }\n`;
+                        css += `#${message.control_id} .toggle-button:hover { background-color: ${colors.hover}; }\n`;
+                    }
+
+                    styleEl.textContent = css;
+                    document.head.appendChild(styleEl);
+                }
 
                 const layersList = document.createElement("div");
                 layersList.className = "layers-list";
@@ -1693,7 +1845,24 @@ if (HTMLWidgets.shinyMode) {
                     link.id = layerId;
                     link.href = "#";
                     link.textContent = layerId;
-                    link.className = "active";
+
+                    // Check if the layer visibility is set to "none" initially
+                    const initialVisibility = map.getLayoutProperty(
+                        layerId,
+                        "visibility",
+                    );
+                    link.className =
+                        initialVisibility === "none" ? "" : "active";
+
+                    // Also hide any associated legends if the layer is initially hidden
+                    if (initialVisibility === "none") {
+                        const associatedLegends = document.querySelectorAll(
+                            `.mapboxgl-legend[data-layer-id="${layerId}"]`,
+                        );
+                        associatedLegends.forEach((legend) => {
+                            legend.style.display = "none";
+                        });
+                    }
 
                     link.onclick = function (e) {
                         const clickedLayer = this.textContent;
@@ -1712,6 +1881,14 @@ if (HTMLWidgets.shinyMode) {
                                 "none",
                             );
                             this.className = "";
+
+                            // Hide associated legends
+                            const associatedLegends = document.querySelectorAll(
+                                `.mapboxgl-legend[data-layer-id="${clickedLayer}"]`,
+                            );
+                            associatedLegends.forEach((legend) => {
+                                legend.style.display = "none";
+                            });
                         } else {
                             this.className = "active";
                             map.setLayoutProperty(
@@ -1719,6 +1896,14 @@ if (HTMLWidgets.shinyMode) {
                                 "visibility",
                                 "visible",
                             );
+
+                            // Show associated legends
+                            const associatedLegends = document.querySelectorAll(
+                                `.mapboxgl-legend[data-layer-id="${clickedLayer}"]`,
+                            );
+                            associatedLegends.forEach((legend) => {
+                                legend.style.display = "";
+                            });
                         }
                     };
 
@@ -1728,7 +1913,25 @@ if (HTMLWidgets.shinyMode) {
                 if (message.collapsible) {
                     const toggleButton = document.createElement("div");
                     toggleButton.className = "toggle-button";
-                    toggleButton.textContent = "Layers";
+
+                    // Use stacked layers icon instead of text if requested
+                    if (message.use_icon) {
+                        // Add icon-only class to the control for compact styling
+                        layersControl.classList.add("icon-only");
+
+                        // More GIS-like layers stack icon
+                        toggleButton.innerHTML = `<svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round">
+                            <polygon points="12 2 2 7 12 12 22 7 12 2"></polygon>
+                            <polyline points="2 17 12 22 22 17"></polyline>
+                            <polyline points="2 12 12 17 22 12"></polyline>
+                        </svg>`;
+                        toggleButton.style.display = "flex";
+                        toggleButton.style.alignItems = "center";
+                        toggleButton.style.justifyContent = "center";
+                    } else {
+                        toggleButton.textContent = "Layers";
+                    }
+
                     toggleButton.onclick = function () {
                         layersControl.classList.toggle("open");
                     };
